SUBROUTINE bootstrap(original_data, nrow, ncol, mode)
    IMPLICIT NONE
    !################################################################################
    ! Performes a bootstrapping on a given data array.
    !
    ! Bootstrapping is performed with replacement.
    ! Thus in the bootstrapped version values can be found more than once and values
    ! can be omitted
    !
    ! author: Florian Boecker   date: 24/11/2011
    ! http://complex-systems.uni-muenster.de/
    ! Licensing:  This code is distributed under the GNU GPL license.
    !
    ! parameters:
    !   name            i/o           datatype      description
    !   ----------------------------------------------------------------------------
    !   original_data   input/output  real(:,:)  - matrix with the original data,
    !                                                 is returned bootstrapped
    !   nrow            input         integer    - number of data rows
    !   ncol            input         integer    - number of data columns
    !   mode            input         integer    - mode in which the array is
    !                                              supposed to be bootstrapped
    !                                              mode < 0 --> row wise
    !                                              mode = 0 --> simultaiusly row
    !                                                           and collumn wise
    !                                              mode > 0 --> collumn wise
    !################################################################################

    !declaration
    INTEGER nrow, ncol  !array dimensions
    INTEGER i, j    !loop control
    REAL, DIMENSION(0:nrow-1,0:ncol-1) :: original_data, bootstraped_data    !data arrays
    DOUBLE PRECISION random_double !random number between 0 and 1, generated by the intrinsic subroutine RANDOM_NUMBER(random)
    INTEGER random_row, random_col  !for random array access, calculated based on random_double
    INTEGER mode    !mode in which the array is supposed to be bootstrapped

    !calculation
    DO i = 0, nrow-1
        DO j = 0, ncol-1
            !row
            IF(mode >= 0) THEN
                CALL RANDOM_NUMBER(random_double)
                random_row = random_double*DBLE(nrow)
                IF(random_row == nrow) random_row = random_row-1
            ELSE
                random_row = i
            ENDIF
            !collumn
            IF(mode <= 0) THEN
                CALL RANDOM_NUMBER(random_double)
                random_col = random_double*DBLE(ncol)
                IF(random_col == nrow) random_col = random_col-1
            ELSE
                random_col = j
            ENDIF
            bootstraped_data(i,j) = original_data(random_row,random_col)
        ENDDO
    ENDDO
    !data swapping
    original_data(:,:) = bootstraped_data(:,:)

END SUBROUTINE bootstrap
