SUBROUTINE comb_part(n, k, y, z, comba, c)
    !#############################################################################
    ! comb_part returns the array comba with c combinations of k numbers in n positions
    ! differing to the original combination on up to z positions
    !
    ! author: Florian Boecker   date: 15/05/2011
    ! http://complex-systems.uni-muenster.de/
    ! Licensing:  This code is distributed under the GNU GPL license.
    !
    ! parameters:
    !   name      i/o     datatype   description
    !   --------------------------------------------------------------------------
    !   n:        input   integer    - the total number of things
    !   k:        input   integer    - the number of things in each combination
    !   y:        input   integer    - the minimum number of factors that should
    !                                  be new in every combination
    !   z:        input   integer    - the maximum number of factors that should
    !                                  be new in every combination
    !   comba:    output  integer    - the returned array with the combinations,
    !                                  should contain at least the c rows and
    !                                  k columns
    !   c:        input   integer    - the number of combinations to be generated
    !#############################################################################

    !prevent implicite variable declaration
    IMPLICIT NONE

    !Declaration #####################################################################
    INTEGER i,j,l   !loop controll
    INTEGER n   !the total number of things
    INTEGER k   !the number of things in each combination
    INTEGER y   !the minimum number of factors that should be new in every combination
    INTEGER z   !the maximum number of factors that should be new in every combination
    INTEGER c   !the number of combinations to be generated
    INTEGER npart !the number of parts
    INTEGER, DIMENSION(c, k) :: comba !array with the combinations
    INTEGER, ALLOCATABLE, DIMENSION(:) :: partindex   !index for the positions that get permuted
    INTEGER, ALLOCATABLE, DIMENSION(:) :: partcomb   !combination of a part
    LOGICAL sizedone
    LOGICAL partdone

    !Initialisation ###################################################################
    i = 1

    !Calculation ######################################################################
    CALL i4vec_indicator(k,comba(i,:))
    i = i+1
    DO j = y, z !iterate sizes
        sizedone = .TRUE.
        ALLOCATE(partindex(j))
        ALLOCATE(partcomb(j))
        DO !iterate parts to be permuted
            CALL comb_next(k, j, partindex, sizedone)
            IF(sizedone) EXIT
            partdone = .TRUE.
            DO  !iterate combinations within the parts
                CALL comb_next(n-k, j, partcomb, partdone)
                IF(partdone) EXIT
                IF(i > c) RETURN
                CALL i4vec_indicator(k,comba(i,:))
                comba(i,partindex) = partcomb(:)+k
                i = i+1
            ENDDO
        ENDDO
        DEALLOCATE(partindex)
        DEALLOCATE(partcomb)
    ENDDO

END SUBROUTINE comb_part
