SUBROUTINE computedistance(rawdata, refset, testset, n, k, ndata, corrdata)
!#############################################################################
! computedistance returns a matrix with the correlation coefficients of given
! test and reference sets
!
! author: Florian Boecker   date: 05/07/2012
! http://complex-systems.uni-muenster.de/
! Licensing:  This code is distributed under the GNU GPL license.
!
! parameters:
!   name      i/o     datatype      description
!   ----------------------------------------------------------------------------
!   rawdata   input   real(:,:)     - the raw data from the data file
!   refset    input   integer(:)    - set of markers to be referenced
!   testset   input   integer(:)    - set of markers to be testet
!   n         input   integer       - the total number of things
!   k         input   integer       - the number of things in the reference set
!   ndata     input   integer       - the number of data records
!   corrdata  output  real(:,:)     - returned matrix with a correlation
!                                     coeffitient for every combination of test
!                                     entry against reference entry
!################################################################################
    IMPLICIT NONE
    ! declaration
    REAL, DIMENSION(ndata,n) :: rawdata   ! the raw data from the data file
    INTEGER, DIMENSION(k) :: refset     ! set of markers to be referenced
    INTEGER, DIMENSION(n-k) :: testset  ! set of markers to be testet
    INTEGER n   ! the total number of things
    INTEGER k   ! the number of things in the reference set
    INTEGER ndata   ! the number of data records
    REAL, DIMENSION(n-k,k) :: corrdata    ! correlation matrix
    INTEGER i, j ! loop-control variables
    REAL r
    ! computation
    DO i = 1, n-k
        DO j = 1, k
            CALL pearsoncorrelation(ndata, rawdata(:,testset(i)), rawdata(:,refset(j)), r)
            corrdata(i,j) = r
        ENDDO
    ENDDO
END SUBROUTINE
