subroutine get_seed ( seed )

!*****************************************************************************80
!
!! GET_SEED returns a seed for the random number generator.
!
!  Discussion:
!
!    The seed depends on the current time, and ought to be (slightly)
!    different every millisecond.  Once the seed is obtained, a random
!    number generator should be called a few times to further process
!    the seed.
!
!  Licensing:
!
!    This code is distributed under the GNU LGPL license.
!
!  Modified:
!
!    27 June 2000
!
!  Author:
!
!    John Burkardt
!
!  Parameters:
!
!    Output, integer ( kind = 4 ) SEED, a pseudorandom seed value.
!
  implicit none

  integer ( kind = 4 ) seed
  real ( kind = 8 ) temp
  character ( len = 10 ) time
  character ( len = 8 )  today
  integer ( kind = 4 ) values(8)
  character ( len = 5 )  zone

  call date_and_time ( today, time, zone, values )

  temp = 0.0D+00

  temp = temp + real ( values(2) - 1, kind = 8 ) / real (  11.0D+00, kind = 8 )
  temp = temp + real ( values(3) - 1, kind = 8 ) / real (  30.0D+00, kind = 8 )
  temp = temp + real ( values(5),     kind = 8 ) / real (  23.0D+00, kind = 8 )
  temp = temp + real ( values(6),     kind = 8 ) / real (  59.0D+00, kind = 8 )
  temp = temp + real ( values(7),     kind = 8 ) / real (  59.0D+00, kind = 8 )
  temp = temp + real ( values(8),     kind = 8 ) / real ( 999.0D+00, kind = 8 )
  temp = temp / real ( 6.0D+00, kind = 8 )

  if ( temp <= 0.0D+00 ) then
    temp = real ( 1.0D+00, kind = 8 ) / real ( 3.0D+00, kind = 8 )
  else if ( 1.0D+00 <= temp ) then
    temp = real ( 2.0D+00, kind = 8 ) / real ( 3.0D+00, kind = 8 )
  end if

  seed = int ( real ( huge ( 1 ), kind = 8 ) * temp )
!
!  Never use a seed of 0 or maximum integer ( kind = 4 ).
!
  if ( seed == 0 ) then
    seed = 1
  end if

  if ( seed == huge ( 1 ) ) then
    seed = seed - 1
  end if

  return
end
