SUBROUTINE pearsoncorrelation(n, x, y, r)
!#############################################################################
! pearsoncorrelation returns a correlation coefficient of two vectors of
! equal length
!
! To avoid a division by zero in the case of uniform verctor, a correlation
! near 0 is randomly generated if the calculted covariance is equal to 0
!
! author: Florian Boecker   date: 05/07/2011
! http://complex-systems.uni-muenster.de/
! Licensing:  This code is distributed under the GNU GPL license.
!
! parameters:
!   name      i/o     datatype      description
!   ----------------------------------------------------------------------------
!   n         input   integer       - number of values in each vector
!   x         input   real(:)       - first vector
!   y         input   real(:)       - second vector
!   r         output  real          - returned correlation coeffitient
!                                           cov(A,B)
!                                     r = -------------
!                                          s(A) * s(B)
!################################################################################
    IMPLICIT NONE
    ! declaration
    INTEGER, INTENT(IN)             :: n      ! number of records
    REAL, DIMENSION(N)              :: x      ! array with the first data series
    REAL, DIMENSION(N)              :: y      ! array with the second data series
    REAL, INTENT(OUT)               :: r      ! correlation coeffitient
    REAL, DIMENSION(n)              :: xdev, ydev, xdevydev, xdevxdev, ydevydev
    REAL                            :: xmn, ymn, COVxy, Sx, Sy
    DOUBLE PRECISION random_double(5) !five random numbers between 0 and 1, generated by the intrinsic subroutine RANDOM_NUMBER(random)

    ! computation
    !   arithmetic mean
    xmn = SUM(x(:)) / REAL(n)
    ymn = SUM(y(:)) / REAL(n)
    !   deviance of the values from the mean
    xdev(:) = x(:) - xmn
    ydev(:) = y(:) - ymn
    !   multiplication of the deviances
    xdevydev(:) = xdev(:) * ydev(:)
    !   covariance
    COVxy = SUM( xdevydev(:)) / REAL(n)
    IF(COVxy .NE. REAL(0)) THEN
        ! squares of the of the deviances
        xdevxdev(:) = xdev(:) * xdev(:)
        ydevydev(:) = ydev(:) * ydev(:)
        ! standard deviation
        Sx = SQRT(SUM(xdevxdev(:)) / REAL(n))
        Sy = SQRT(SUM(ydevydev(:)) / REAL(n))
        ! correlation coeffitient
        r = COVxy / (Sx * Sy)
    ELSE ! avoid a correlation of zero because linfit is unable to handle it
        CALL RANDOM_NUMBER(random_double(:))
        r = (random_double(4)-DBLE(0.5))*1.0E-7
    ENDIF

END SUBROUTINE pearsoncorrelation
