SUBROUTINE read_tma_data_file(datafilename, ncol, nrow, nfactor, nrecord, mapping, n, header, data)
!#################################################################################
! reads a the csv-tma-data-file guided by a mapping file and returns its data
! in a real-array
!
! author: Florian Boecker   date: 19/04/2012
! http://complex-systems.uni-muenster.de/
! Licensing:  This code is distributed under the GNU GPL license.
!
! parameters:
!   name            i/o     datatype   description
!   ------------------------------------------------------------------------------
!   datafilename    input   character(:)    - the complete path of the file to be
!                                             read in
!   ncol            input   integer         - the number of columns in the file
!   nrow            input   integer         - the number of rows in the file
!   nfactor         input   integer         - the number of factors in the file
!   nrecord         input   integer         - the number of data records
!   mapping         input   integer(:)      - mapping of the data
!   n               input   integer         - number of factors to be incoperated and also the
!                                             length of the mapping
!   header          output  character(:)    - names of the factors
!   data            output  real            - the data of the file in an array
!#################################################################################
    IMPLICIT  NONE
    ! Declaration
    CHARACTER (len=200) :: datafilename     ! path and name of the data file
    CHARACTER (len=10) :: header(n)         ! names of the factors
    CHARACTER (len=10) :: line(ncol)        ! data record buffer
    INTEGER i, j                            ! loop control
    INTEGER ncol                            ! number of columns in the file
    INTEGER nrow                            ! number of rows in the file
    INTEGER nfactor                         ! number of factors in the data
    INTEGER nrecord                         ! number of data records
    INTEGER n
    INTEGER unitnumber                      ! input stream ID for the data file
    INTEGER :: mapping(n)                   ! mapping of the data
    REAL :: data(nrecord,n)                 ! array for the data

    ! Initialisation
    CALL get_unit(unitnumber)
    OPEN(unit = unitnumber, file = datafilename)

    ! Read the header
    IF(nrecord < nrow) THEN
        READ (unitnumber,*) line(:)
        IF(nfactor < ncol) THEN
            header(:) = line(mapping(:)+1)
        ELSE
            header(:) = line(mapping(:))
        ENDIF
    ENDIF

    ! Read in the Data
    DO i = 1, nrecord
        READ (unitnumber,*) line(:) ! Read the Data as Character
        DO j = 1, n
            IF(nfactor < ncol) THEN
                READ(line(mapping(j)+1),*) data(i,j)
            ELSE
                READ(line(mapping(j)),*) data(i,j)
            ENDIF
        ENDDO
    ENDDO

    ! Close the File
    CLOSE (unitnumber)
END SUBROUTINE read_tma_data_file
