SUBROUTINE ssq(data, nsamples, nref, ntest, ssq_best, ssq_g, test_order)
! ###############################################################
! ssq
!
! returns the minimal global SSQ for all test set permutations,
! the optimal order of the test factors, and for each reference
! factor the SSQ for the optimal rank order.
!
! author: Kathrin Poos   date: 03/12/2010
! http://complex-systems.uni-muenster.de/
! Licensing:  This code is distributed under the GNU GPL license.
! ###############################################################
    IMPLICIT NONE
    ! global variables to use outside the subroutine
    INTEGER nsamples, nref, ntest, test_order(nsamples,ntest)
    REAL data(0:((ntest*nsamples)-1),0:(nref-1)), ssq_best(nsamples,nref), ssq_g(nsamples)
    !integer,dimension(0:(nsamples-1),0:(ntest-1))::test_order
    !real,dimension(0:((ntest*nsamples)-1),0:(nref-1))::data
    !real,dimension(0:(nsamples-1),0:(nref-1))::ssq_best
    !real,dimension(0:(nsamples-1))::ssq_g

    ! local variables internal use only
    INTEGER,DIMENSION(0:(ntest-1))::perm_test
    INTEGER,DIMENSION(0:ntest)::iter_contr
    INTEGER,DIMENSION(0:(nsamples-1))::indices_data
    INTEGER,DIMENSION(0:(ntest-1))::test_set
    REAL tmp, ssq_s, ssq_sum
    REAL,DIMENSION(0:(nref-1))::omat
    REAL,DIMENSION(0:(ntest-1),0:(nref-1))::cor
    REAL,DIMENSION(0:(ntest-1))::part_cor

    ! perm_test = a, iter_contr = p

    ! local counter variables
    INTEGER i, j, k, l, counter

    ! fills the vector indices_data to get the
    ! indices of the different samples
    i = 0
    counter = 0
    ! do while counter i is smaller than the number of
    ! samples
    DO WHILE(i < nsamples)
        indices_data(i) = counter
        counter = counter+ntest
        i = i+1
    END DO

    test_set = (/ (I,I=1,ntest,1) /)
    ! set index of test factors into perm_test
    perm_test = (/ (I,I=0,(ntest-1),1) /)

    !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    !! start permuting the test factors for all possible permutations   !!
    !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

    ! set index of reversed test factors into permrev_test
    !permrev_test = ntest:1
    ! set indices of the iteration control variable iter_contr
    ! controls iteration and the index boundary for i
    iter_contr = (/ (I,I=0,ntest,1) /)
    ! counter for the do while loop
    i = 0
    counter = 0
    ! do while i is smaller than number of test factors
    DO WHILE(i < ntest)
        iter_contr(i) = iter_contr(i)-1 	! decrease index "weight" for i by one
        j = mod(i,2) * iter_contr(i) 		! IF i is odd then j =iter_perm(i) otherwise j = 0
        tmp = perm_test(j)					! swap perm_test(j) with perm_test(i)
        perm_test(j) = perm_test(i)
        perm_test(i) = tmp					! perm_test has a new order, 1 permutation round

        !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
        !! start computation of the optimal test factor order with the minimum   !!
        !! global sum															 !!
        !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

        k = 0
        ! do while counter k is smaller than the number of
        ! samples
        DO WHILE(k < nsamples)
            cor = data(indices_data(k):(indices_data(k)+(ntest-1)), 0:(nref-1))
            l = 0
            ! do while counter l is smaller than the number of
            ! reference factors
            DO WHILE(l < nref)
                part_cor = cor(perm_test, l)
                CALL linfit(part_cor,test_set,ntest,ssq_s)
                omat(l) = ssq_s
                l = l+1
            END DO

            IF(counter == 0) THEN
                ssq_best((k+1),1:nref) = omat
                ssq_g(k+1) = sum(omat)
                test_order((k+1),1:ntest) = test_set(perm_test)
                IF(k == (nsamples-1)) THEN
                    counter = 1
                ENDIF
            ENDIF

            IF(counter > 0) THEN
                ssq_sum = sum(omat)
                !print *, omat
                !print *, ssq_sum
                ! if sum of new computed sum of squares smaller than
                ! formerly computed go into the statement
                IF(ssq_sum < ssq_g(k+1)) THEN
                    ssq_best((k+1),1:nref) = omat
                    ssq_g(k+1) = ssq_sum
                    test_order((k+1),1:ntest) = test_set(perm_test)
                ENDIF
            ENDIF
            k = k+1
        END DO
        !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
        !! stop computation of the optimal test factor order with the minimum    !!
        !! global sum															 !!
        !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
        i = 1  					    		!reset index i to 1 (assumed)
        DO WHILE(iter_contr(i) == 0)        ! while (iter_perm(i) == 0) do
            iter_contr(i) = i      			! reset iter_contr(i) zero value
            i = i+1             			! set new index value for i (increase by one)
        END DO
    END DO
    !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    !! stop permuting the test factors for all possible permutations    !!
    !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    RETURN
END SUBROUTINE ssq
