subroutine task_division(task_number, p, task_distribution)
! ###############################################################
! task_division
!
!
! author: Florian Boecker   date: 2012
! http://complex-systems.uni-muenster.de/
! Licensing:  This code is distributed under the GNU GPL license.
! ###############################################################

  implicit none

  integer ( kind = 4 ) i_hi
  integer ( kind = 4 ) i_lo
  integer ( kind = 4 ) i4_div_rounded
  integer ( kind = 4 ) p
  integer ( kind = 4 ) proc
  integer ( kind = 4 ) proc_number
  integer ( kind = 4 ) proc_remain
  integer ( kind = 4 ) task_number
  integer ( kind = 4 ) task_proc
  integer ( kind = 4 ) task_remain
  INTEGER, DIMENSION(p,2) :: task_distribution

  i_hi = 0

  task_remain = task_number
  proc_remain = p

  do proc = 1, p

    task_proc = i4_div_rounded ( task_remain, proc_remain )

    proc_remain = proc_remain - 1
    task_remain = task_remain - task_proc

    i_lo = i_hi + 1
    i_hi = i_hi + task_proc

    task_distribution(proc,1) = i_lo
    task_distribution(proc,2) = i_hi

  end do

  return
end

function i4_div_rounded ( a, b )

!*****************************************************************************80
!
!! I4_DIV_ROUNDED computes the rounded result of I4 division.
!
!  Discussion:
!
!    This routine computes C = A / B, where A, B and C are integers
!    and C is the closest integer value to the exact real result.
!
!  Licensing:
!
!    This code is distributed under the GNU LGPL license.
!
!  Modified:
!
!    23 October 2011
!
!  Author:
!
!    John Burkardt
!
!  Parameters:
!
!    Input, integer ( kind = 4 ) A, the number to be divided.
!
!    Input, integer ( kind = 4 ) B, the divisor, or the number of parts.
!
!    Output, integer ( kind = 4 ) I4_DIV_ROUNDED, the rounded result
!    of the division.
!
  implicit none

  integer ( kind = 4 ) a
  integer ( kind = 4 ) a_abs
  integer ( kind = 4 ) b
  integer ( kind = 4 ) b_abs
  integer ( kind = 4 ) i4_div_rounded
  integer ( kind = 4 ), parameter :: i4_huge = 2147483647
  integer ( kind = 4 ) value

  if ( a == 0 .and. b == 0 ) then

    value = i4_huge

  else if ( a == 0 ) then

    value = 0

  else if ( b == 0 ) then

    if ( a < 0 ) then
      value = - i4_huge
    else
      value = + i4_huge
    end if

  else

    a_abs = abs ( a )
    b_abs = abs ( b )

    value = a_abs / b_abs
!
!  Round the value.
!
    if ( ( 2 * value + 1 ) * b_abs < 2 * a_abs ) then
      value = value + 1
    end if
!
!  Set the sign.
!
    if ( ( a < 0 .and. 0 < b ) .or. ( 0 < a .and. b < 0 ) ) then
      value = - value
    end if

  end if

  i4_div_rounded = value

  return
end
